<?php
// $Id: views_handler_filter_dynamic_fields.inc,v 1.1 2010/09/01 07:35:07 girishmuraly Exp $
/**
 * @file
 * Views dynamic fields filter handler
 */

/**
 * Simple filter to pick and choose the fields to show in the view.
 */
class views_handler_filter_dynamic_fields extends views_handler_filter {

  var $no_single = TRUE;

  function has_extra_options() { return TRUE; }

  function option_definition() {
    $options = parent::option_definition();
    $options['reverse'] = array('default' => 0);
    $options['reverse_label'] = array('default' => '');
    return $options;
  }

  /**
   * Provide a list of all fields that are for display
   */
  function value_form(&$form, &$form_state) {
    $form['value']['#tree'] = TRUE;
    $all_fields = array();
    foreach ($this->view->field as $field_name => $field) {
      if (!isset($_SESSION[$this->view->name]['perm_exc']) && $field->options['exclude']) {
        // Fields already marked as 'exclude from display' in the original view should be omitted
        $_SESSION[$this->view->name]['perm_exc'][$field_name] = $field_name;
        continue;
      }
      if (!isset($_SESSION[$this->view->name]['perm_exc'][$field_name])) {
        $all_fields[$field_name] = $field->label();
        $field_names[] = $field_name;
      }
    }

    $this->options['expose']['single'] = ''; // otherwise views_handler_filter will change this to use 'select'
    if($form_state['exposed']) {
      $form['value'] = array(
        '#type' => 'checkboxes',
        '#no_convert' => TRUE,
        '#title' => $this->options['expose']['label'] ? t($this->options['expose']['label']) : t('Fields'),
        '#default_value' => $field_names,
        '#options' => $all_fields,
        '#description' => t('Select fields for the view - these will be shown by default'),
      );
      if ($this->options['reverse']) {
        $form['reverse'] = array(
          '#type' => 'checkbox',
          '#default_value' => $this->options['reverse'],
          '#title' => $this->options['reverse_label'] ? t($this->options['reverse_label']) : t('Show fields to be excluded'),
          '#description' => t('Use the field selection to exclude them from the view'),
        );
      }
    }
    else {
      // Ensure there is something in the 'value'.
      $form['value'] = array(
        '#type' => 'value',
        '#value' => NULL
      );
    }
  }

  /**
   * Provide an 'exclusion mode' option
   * @see sites/all/modules/contrib/views/includes/views_handler#extra_options_form()
   */
  function extra_options_form(&$form, &$form_state) {
    $form['reverse'] = array(
      '#type' => 'checkbox',
      '#default_value' => $this->options['reverse'],
      '#title' => t('Provide exclusion mode'),
      '#description' => t('Provide a checkbox to reverse the selection from inclusion to exclusion'),
    );
    $form['reverse_label'] = array(
      '#type' => 'textfield',
      '#default_value' => $this->options['reverse_label'],
      '#title' => t('Label for exclusion fields options'),
      '#size' => 40,
    );
  }

  /**
   * Exclude fields from the view before the query is run
   * @see sites/all/modules/contrib/views/includes/views_handler#pre_query()
   */
  function pre_query() {
    $exposed_input = $this->view->get_exposed_input();
    // Get all fields
    foreach ($this->view->field as $field_name => $field) {
      $all_fields[$field_name] = $field->label();
      $field_names[] = $field_name;
    }
    if (!empty($exposed_input[$this->options['expose']['identifier']])) {
      // Exclude these fields

      // If exclusion mode is selected
      if ($_REQUEST['reverse']) {
        $fields_to_exclude = $exposed_input[$this->options['expose']['identifier']];
      }
      else {
        $fields_to_exclude = array_diff($field_names, $exposed_input[$this->options['expose']['identifier']]);
      }
      // Mark the fields to be excluded
      foreach ($fields_to_exclude as $field_to_exclude) {
        $this->view->field[$field_to_exclude]->options['exclude'] = 1;
      }
    }
  }

  /**
   * Do nothing in the query. This function needs to be defined and left to do nothing.
   * @see sites/all/modules/contrib/views/handlers/views_handler_filter#query()
   */
  function query() {
  }

}
